
transcript

again me
n
again
again me
put sweater in s
drop sweater
put ladder on ladder
put ladder in ladder
go n
go sweater
go
get
x orb
frob it
x ladder
frob ladder
frob it
pronouns
get ladder and
kiss ladder
open it
type abc on ladder
type abc into ladder
close close door
open close door
i.i
open torch and ladder
i blue
open all
put all
take torch.
put all
take torch and john
open box
get all from box
i
put all
put all except
put all except in box
put all except book in box
take all from box
put all except red book in box
i
drop all
get red book, xyzzy and torch
get xyzzy and torch
get xyzzy
x green
get green
read green
purloin pink book
i
take book
drop pink
take all
quit
yes

